@@ -1,3 +1,5 @@
+import logging
+
 from django.conf import settings
 from rest_framework.permissions import BasePermission
 
@@ -5,6 +7,9 @@ from rest_framework.permissions import BasePermission
 __all__ = ["BlockSchemaAccess"]
 
 
+logger = logging.getLogger(__name__)
+
+
 class BlockSchemaAccess(BasePermission):
     """Block schema access from OPTIONS request when not in DEBUG mode."""
 
