@@ -1,7 +1,7 @@
+import logging
 import re
-from typing import Any, Callable, Dict, List, Tuple, Type, Union
 
-from django.urls import include, path, re_path
+from django.urls import include, re_path
 from django.urls.resolvers import URLPattern, URLResolver
 from rest_framework.routers import DefaultRouter
 from rest_framework.schemas import DefaultSchema
@@ -9,43 +9,54 @@ from rest_framework.urlpatterns import format_suffix_patterns
 from rest_framework.views import APIView
 from rest_framework.viewsets import ViewSetMixin
 
+from .typing import Any, Callable, Dict, List, Optional, Tuple, Type, Union, UrlsType, ViewType
 from .views import APIRootView as RootView
 
 
 __all__ = ["APIRouter"]
 
 
-UrlsType = list[Union[URLResolver, URLPattern]]
-ViewType = Union[Type[APIView], Type[ViewSetMixin]]
+logger = logging.getLogger(__name__)
 
 
 class APIRouter(DefaultRouter):
     """Router that will show APIViews in API root."""
 
-    registry: List[Tuple[str, Type[APIView], str, dict]]
+    registry: List[Tuple[str, Type[APIView], str, Dict[str, Any]]]
     APIRootView: APIView = RootView
 
-    def __init__(self, name: str = None, docstring: str = None, **kwargs):
-        self._navigation_routes: Dict[str, "APIRouter"] = {}
+    def __init__(
+        self,
+        *,
+        name: Optional[str] = None,
+        docstring: Optional[str] = None,
+        show_in_shema: bool = False,
+        ignore_model_permissions: bool = False,
+        navigation_routes: Optional[Dict[str, "APIRouter"]] = None,
+        **kwargs: Any,
+    ):
+        self._navigation_routes: Dict[str, "APIRouter"] = navigation_routes or {}
 
         name = name if name is not None else self.APIRootView.__name__
         self._root_view: Type[self.APIRootView] = type(name, (self.APIRootView,), {})  # noqa
         self._root_view.__doc__ = docstring if docstring else self.APIRootView.__doc__
-        self._root_view.schema = DefaultSchema() if kwargs.get("show_in_shema", False) else None
-        self._root_view._ignore_model_permissions = kwargs.get("ignore_model_permissions", False)
+        self._root_view.schema = DefaultSchema() if show_in_shema else None
+        self._root_view._ignore_model_permissions = ignore_model_permissions
 
         self.root_view_name: str = name
         super().__init__(**kwargs)
 
     def register(self, prefix: str, view: ViewType, basename: str = None, **kwargs: Any):  # pylint: disable=W0237
         if basename is None:
-            basename = self.get_default_basename(view)
+            basename = self.get_default_basename(view)  # pragma: no cover
 
+        # Construct default values for regex parts
         params = {key: "..." for key in re.compile(prefix).groupindex}
         params.update(kwargs)
         self.registry.append((prefix, view, basename, params))
 
-        if hasattr(self, "_urls"):
+        # Invalidate the urls cache
+        if hasattr(self, "_urls"):  # pragma: no cover
             del self._urls
 
     @property
@@ -58,13 +69,13 @@ class APIRouter(DefaultRouter):
     def navigation_routes(self, value: Dict[str, "APIRouter"]):
         self._navigation_routes = value
 
-    def get_routes(self, viewset: type[Union[ViewSetMixin, APIView]]):
+    def get_routes(self, viewset: Type[Union[ViewSetMixin, APIView]]):
         if issubclass(viewset, ViewSetMixin):
             return super().get_routes(viewset)
-        return []
+        return []  # pragma: no cover
 
     def get_api_root_view(self, api_urls: UrlsType = None) -> Callable[..., Any]:
-        api_root_dict: Dict[str, Tuple[str, dict[str, Any]]] = {}
+        api_root_dict: Dict[str, Tuple[str, Dict[str, Any]]] = {}
         list_name = self.routes[0].name
 
         for prefix, viewset, basename, kwargs in self.registry:
@@ -74,13 +85,13 @@ class APIRouter(DefaultRouter):
                 api_root_dict[prefix] = basename, kwargs
 
         for basename in self.navigation_routes:
-            api_root_dict[fr"{basename}"] = basename, {}
+            api_root_dict[rf"{basename}"] = basename, {}
 
         return self._root_view.as_view(api_root_dict=api_root_dict)
 
     def format_regex(self, url: str, prefix: str, lookup: str = "") -> str:
         regex = url.format(prefix=prefix, lookup=lookup, trailing_slash=self.trailing_slash)
-        if not prefix and regex[:2] == "^/":
+        if not prefix and regex[:2] == "^/":  # pragma: no cover
             regex = "^" + regex[2:]
         return regex
 
@@ -119,6 +130,6 @@ class APIRouter(DefaultRouter):
 
         for basename, router in self.navigation_routes.items():
             router.root_view_name = basename
-            urls.append(path(f"{basename}/", include(router.urls)))
+            urls.append(re_path(rf"^{basename}/$", include(router.urls)))
 
         return urls
