@@ -1,5 +1,4 @@
 import logging
-from typing import Any, Dict, Tuple
 
 from django.urls import NoReverseMatch
 from rest_framework.request import Request
@@ -7,6 +6,8 @@ from rest_framework.response import Response
 from rest_framework.reverse import reverse
 from rest_framework.views import APIView
 
+from .typing import Any, Dict, Tuple
+
 
 __all__ = ["APIRootView"]
 
@@ -17,7 +18,7 @@ logger = logging.getLogger(__name__)
 class APIRootView(APIView):
     """Welcome! This is the API root."""
 
-    api_root_dict: Dict[str, Tuple[str, dict[str, Any]]] = {}
+    api_root_dict: Dict[str, Tuple[str, Dict[str, Any]]] = {}
 
     authentication_classes = []
     permission_classes = []
